@@ -16,6 +16,7 @@
 #   \item{tags}{A @character @vector of tags to be used for this file set.
 #      The string \code{"*"} indicates that it should be replaced by the
 #      tags part of the file set pathname.}
+#   \item{depth}{An non-negative @integer.}
 #   \item{alias}{A @character string specifying a name alias overriding the
 #      name inferred from the pathname.}
 #   \item{...}{Not used.}
@@ -31,7 +32,7 @@
 # 
 # @author
 #*/###########################################################################
-setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", alias=NULL, ..., .onUnknownArgs=c("error", "warning", "ignore")) {
+setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", depth=NULL, alias=NULL, ..., .onUnknownArgs=c("error", "warning", "ignore")) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Validate arguments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -46,6 +47,11 @@ setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", alias=NULL
     throw("Argument 'files' is of unknown type: ", mode(files)[1]);
   }
 
+  # Arguments 'depth':
+  if (!is.null(depth)) {
+    depth <- Arguments$getInteger(depth, range=c(0,32));
+  }
+
   # Arguments '.onUnknownArgs':
   .onUnknownArgs <- match.arg(.onUnknownArgs);
 
@@ -66,6 +72,7 @@ setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", alias=NULL
 
   this <- extend(Object(), c("GenericDataFileSet", uses("FullNameInterface")),
     files = as.list(files),
+    .depth = depth,
     .alias = NULL,
     .tags = NULL
   );
@@ -121,6 +128,12 @@ setMethodS3("as.character", "GenericDataFileSet", function(x, ...) {
   # Full names of file set
   s <- c(s, sprintf("Full name: %s", getFullName(this)));
 
+  # Subdirectories(?)
+  subdirs <- getSubdirs(this, default=NULL);
+  if (length(subdirs) > 0) {
+    s <- c(s, sprintf("Subpath: %s", subdirs));
+  }
+
   # Number of files in set
   n <- nbrOfFiles(this);
   s <- c(s, sprintf("Number of files: %d", n));
@@ -336,6 +349,57 @@ setMethodS3("getPath", "GenericDataFileSet", function(this, ...) {
 })
 
 
+setMethodS3("getDepth", "GenericDataFileSet", function(this, default=0L, ...) {
+  # Argument 'default':
+  if (!is.null(default)) {
+    default <- Arguments$getInteger(default, range=c(0,32));
+  }
+
+  depth <- this$.depth;
+  if (is.null(depth)) {
+    depth <- default;
+  }
+  depth;
+}, private=TRUE)
+
+
+setMethodS3("setDepth", "GenericDataFileSet", function(this, depth=0L, ...) {
+  # Argument 'depth':
+  if (!is.null(depth)) {
+    depth <- Arguments$getInteger(depth, range=c(0,32));
+  }
+
+  this$.depth <- depth;
+
+  invisible(this);
+}, private=TRUE)
+
+
+setMethodS3("getSubdirs", "GenericDataFileSet", function(this, collapse="/", ...) {
+  if (!is.null(collapse)) {
+    collapse <- Arguments$getCharacter(collapse);
+  }
+
+  depth <- getDepth(this, ...);
+  if (is.null(depth)) {
+    return(NULL);
+  }
+
+  path <- getPath(this);
+  dirs <- character(length=depth);
+  for (dd in seq(length=depth)) {
+    dirs[dd] <- basename(path);
+    path <- dirname(path);
+  }
+  dirs <- rev(dirs);
+
+  if (length(dirs) > 1 && !is.null(collapse)) {
+    dirs <- paste(dirs, collapse=collapse);
+  }
+
+  dirs;
+}, protected=TRUE) # getSubdirs()
+
 
 ###########################################################################/**
 # @RdocMethod length
@@ -1160,7 +1224,7 @@ setMethodS3("extract", "GenericDataFileSet", function(this, files, ..., onMissin
 #   @seeclass
 # }
 #*/###########################################################################
-setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=NULL, recursive=FALSE, fileClass=getFileClass(static), ..., .validate=FALSE, verbose=FALSE) {
+setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=NULL, recursive=FALSE, depth=0L, fileClass=getFileClass(static), ..., .validate=FALSE, verbose=FALSE) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Validate arguments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1187,6 +1251,7 @@ setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=
 
   verbose && enter(verbose, "Defining an ", class(static)[1], " object from files");
   verbose && cat(verbose, "Path: ", path);
+  verbose && cat(verbose, "Depth: ", depth);
   verbose && cat(verbose, "Pattern: ", pattern);
   verbose && cat(verbose, "File class: ", class(dfStatic)[1]);
 
@@ -1252,6 +1317,9 @@ setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=
   update2(set, ..., verbose=less(verbose, 5));
   verbose && exit(verbose);
 
+  # Set depth
+  setDepth(set, depth);
+
   # Validate?
   if (.validate) {
     validate(set, verbose=less(verbose, 5));
@@ -1637,6 +1705,18 @@ setMethodS3("byName", "GenericDataFileSet", function(static, name, tags=NULL, su
   verbose && cat(verbose, "Name: ", name);
   verbose && cat(verbose, "Tags: ", paste(tags, collapse=","));
 
+  # Record the "depth"/"subdirs".
+  if (length(subdirs) > 0) {
+    subdirsT <- unlist(strsplit(subdirs, split="/\\\\"), use.names=FALSE);
+    depth <- length(subdirsT);
+  } else {
+    depth <- 0L;
+  }
+
+  verbose && printf(verbose, "Subpath [%d]: %s\n", 
+                    depth, paste(subdirs, collapse="/"));
+
+
   suppressWarnings({
     paths <- findByName(static, name=name, tags=tags, subdirs=subdirs, 
              paths=paths, firstOnly=FALSE, mustExist=TRUE, verbose=verbose);
@@ -1652,7 +1732,7 @@ setMethodS3("byName", "GenericDataFileSet", function(static, name, tags=NULL, su
     verbose && cat(verbose, "Path: ", path);
 
     suppressWarnings({
-      res <- byPath(static, path=path, ..., verbose=verbose);
+      res <- byPath(static, path=path, depth=depth, ..., verbose=verbose);
     });
 
     if (!is.null(res)) {
@@ -1791,9 +1871,11 @@ setMethodS3("update2", "GenericDataFileSet", function(this, ...) {
 #   @seeclass
 # }
 #*/###########################################################################
-setMethodS3("getDefaultFullName", "GenericDataFileSet", function(this, parent=0, ...) {
+setMethodS3("getDefaultFullName", "GenericDataFileSet", function(this, parent=getDepth(this, default=NULL), ...) {
   # Argument 'parent':
-  parent <- Arguments$getInteger(parent, range=c(0,32));
+  if (!is.null(parent)) {
+    parent <- Arguments$getInteger(parent, range=c(0,32));
+  }
 
   # The name of a file set is inferred from the pathname of the directory
   # of the set assuming path/to/<fullname>/<something>/<subdir>/
@@ -1805,10 +1887,12 @@ setMethodS3("getDefaultFullName", "GenericDataFileSet", function(this, parent=0,
     return(naValue);
   }
 
-  while (parent > 0) {
-    # path/to/<fullname>/<something>
-    path <- dirname(path);
-    parent <- parent - 1;
+  if (!is.null(parent)) {
+    while (parent > 0) {
+      # path/to/<fullname>/<something>
+      path <- dirname(path);
+      parent <- parent - 1;
+    }
   }
 
   # <fullname>
@@ -1918,6 +2002,14 @@ setMethodS3("fromFiles", "GenericDataFileSet", function(static, ...) {
 
 ############################################################################
 # HISTORY:
+# 2011-09-11
+# o Added argument 'default' to getDepth().
+# o BUG FIX: GenericDataFileSet$byName(..., subdirs) would throw 'Error
+#   in strsplit(subdirs, split = "/\\")' iff subdirs != NULL.
+# 2011-07-25
+# o Now getDefaultFullName() of GenericDataFileSet utilizes getDepth().
+# o Now as.character() of GenericDataFileSet displays the subdirs.
+# o Added argument 'depth' to GenericDataFileSet.
 # 2011-05-23
 # o Added argument '.fileClass' to appendFiles() for GenericDataFileSet.
 # 2011-05-16
